////////////////////////////////////////////////////////////////////////////////
//  Project: Pong Game Clone
//  Author: Vladeta Stojanovic (vladeta_stojanovic@yahoo.co.uk)
//  File: core.hpp
//  Description: This file contains the sprite handler struct,
//  as well as all the core function prototypes.
//  Version: v0.1
//
//  Date (last revision): March 16, 2009
//  Date (creation): March 30, 2009
//
//
//  Revision history notes:
//  ---Date---|---Revision---|---Revisor---|
//
//  Credits:  Portions of this code were based on the examples form the book
//            "Programming The Nintendo Game Boy Advance: The Unofficial Guide"
//            by Jonathan S. Harbour (http://www.jharbour.com).
//            Paddle collision detection code based on the Pong game code
//            by Aaron Cox (http://www.aaroncox.net).
//            Additional code and examples are derived from the code presented
//            in TONC v1.4 by J Vijn (http://www.coranac.com/projects/#tonc).
//
//  License: You are allowed to use the code in this file for presonal,
//           scholarship and/or non-commercial purpuses. This program code
//           comes with NO WARRANTY.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _CORE_H_
#define _CORE_H_

#include <stdlib.h>  //used for the rand() function
#include <string.h> //used for memcpy function

//sprite structs
typedef struct tag_sprite
{
unsigned short attribute0;
unsigned short attribute1;
unsigned short attribute2;
unsigned short attribute3;
}Sprite,*pSprite;

//sprite handler struct
typedef struct tag_sprite_handler
{
    int alive; //is the sprite being drawn?
    int x, y; // x & y drawing coords for the sprite
    int dir_x, dir_y; //x & y stepping coordinates for the sprite
    int size; //size of the sprite
}Sprite_handler;

//create and export an array of 128 sprites equal to OAM
extern Sprite sprites[128];

//create and export an array of 128 sprites with the sprite handling attributes
extern Sprite_handler game_sprites[128];

//create an array of button key press values
extern bool buttons[10];

//core function prototypes
void vsync(); //vsync
void init_sprites(int num, bool paddle, int x, int y, int size, int color, int tile_index); //initialize the sprite within the sprite handler array
void hide_sprites(); //move sprites off the screen
void move_sprite(int num); //change x and y attributes of desired sprite
void update_oam(); //update oam function
void timer(int time); //game timer
void check_buttons(); //check if button is pressed
bool button_pressed(int button); //press button function

#endif
