////////////////////////////////////////////////////////////////////////////////
//  Project: Pong Game Clone
//  Author: Vladeta Stojanovic (vladeta_stojanovic@yahoo.co.uk)
//  File: defs.hpp
//  Description: Thsi file contatins all of the core defines for interfacing the program
//  with the GBA hardware, as well as defines for gameplay values.
//  Version: v0.1
//
//  Date (last revision): March 16, 2009
//  Date (creation): March 30, 2009
//
//
//  Revision history notes:
//  ---Date---|---Revision---|---Revisor---|
//
//  Credits:  Portions of this code were based on the examples form the book
//            "Programming The Nintendo Game Boy Advance: The Unofficial Guide"
//            by Jonathan S. Harbour (http://www.jharbour.com).
//            Paddle collision detection code based on the Pong game code
//            by Aaron Cox (http://www.aaroncox.net).
//            Additional code and examples are derived from the code presented
//            in TONC v1.4 by J Vijn (http://www.coranac.com/projects/#tonc).
//
//  License: You are allowed to use the code in this file for presonal,
//           scholarship and/or non-commercial purpuses. This program code
//           comes with NO WARRANTY.
//
///////////////////////////////////////////////////////////////////////////////

//Add all additional game defines here

#ifndef _DEFS_HPP_
#define _DEFS_HPP_

//typedefs
typedef unsigned short volatile vu16;
typedef unsigned short u16;

//pointer to video buffer
extern u16 *video_buffer;

//create a pointer to the control input register
extern vu16 *BUTTONS;

//create a pointer to OAM for use with mode 3, which starts at 0x6010000 + 8192
extern u16 *SpriteData3;

//macro to change the video mode
#define SetMode(mode) REG_DISPCNT = (mode)

//define some video addresses
#define REG_DISPCNT *(vu16*)0x4000000
#define BGPaletteMem ((u16*)0x5000000)
#define REG_VCOUNT *(vu16*)0x4000006
#define REG_DISPSTAT *(vu16*)0x4000004

//button defines
#define BUTTON_A 1
#define BUTTON_B 2
#define BUTTON_SELECT 4
#define BUTTON_START 8
#define BUTTON_RIGHT 16
#define BUTTON_LEFT 32
#define BUTTON_UP 64
#define BUTTON_DOWN 128
#define BUTTON_R 256
#define BUTTON_L 512

//define object attribute memory state address
#define SpriteMem ((u16*)0x7000000)

//define object attribute memory image address
#define SpriteData ((u16*)0x6010000)

//define object attribute memory palette address
#define SpritePal ((u16*)0x5000200)

//misc sprite constants
#define OBJ_MAP_2D 0x0 //sprite tile data in 2d mode is an actual iamge
#define OBJ_MAP_1D 0x40 //chopped up into an 8x8 tile
#define OBJ_ENABLE 0x1000 //enalbe sprite
#define BG2_ENABLE 0x400 //enable background 2

//attribute0 stuff
#define ROTATION_FLAG 0x100 //affine-specific attribute
#define SIZE_DOUBLE 0x200 //double size sprite scaling
#define MODE_NORMAL 0x0
#define MODE_TRANSPARENT 0x400
#define MODE_WINDOWED 0x800
#define MOSAIC 0x1000
#define COLOR_16 0x0000
#define COLOR_256 0x2000
#define SQUARE 0x0
#define TALL 0x4000
#define WIDE 0x8000

//attribute1 stuff
#define ROTDATA(n) ((n) << 9)
#define HORIZONTAL_FLIP 0x1000
#define VERTICAL_FLIP 0x2000
#define SIZE_8 0x0
#define SIZE_16 0x4000
#define SIZE_32 0x8000
#define SIZE_64 0xC000

//attribute2 stuff
#define PRIORITY(n) ((n) << 10)
#define PALETTE(n) ((n) << 12)

//game defines
#define BALL_X_Y_SPEED 2
#define PADDLE_Y_SPEED 2
#define NUM_BALLS 1
#define NUM_PADDLES 2 //change to 2 later

#endif


